/************** BEST VIEWED IN A CODE EDITOR 80 COLUMNS WIDE *******************
*
* ToughBuilt ClipTech Mount
* Benjamen Johnson <workshop.electronsmith.com>
* 20201108
*
* openSCAD Version: 2019.05 
* Licesnse: Creative Commons - Attribution - Share Alike
*           https://creativecommons.org/licenses/by-sa/4.0/
*******************************************************************************/
/*[Hidden]*/
version = "V2.0";

/*[Choose Features]*/
// Choose the part to print
type = 0; //[0:Universal Clip,1:Rail Clamp]

// Locking Foot Connector?
foot_connector = 1;//[0:No, 1: Yes]

//Do you want the wall mount holes?
wall_mount_holes = 1;//[0:No, 1: Yes]

// Do you want the rail mount holes?
rail_mount_holes = 1;//[0:No, 1: Yes]

// Do you need set screws to keep the holder from sliding on the rail?
set_screw_nuts = 1;//[0:No, 1: Yes]

/*[Clip Parameters]*/
/*******************************************************************************
*        
*         |- top-| 
*          ------      ---
*        /        \     |
*       /          \    clip_height
*      /            \   |
*     /              \  |
*     ---------------- ---
*    |---- bottom ----|
*
*         
*          ------     --- top_gap
*        /  ----  \   ---  
*       /  |    |  \
*      /   |    |   \
*     /    |    |    \
*     ----------------
*          |    |
*        base_width
*
*******************************************************************************/
// Length of the base of the trapezoid part of the clip
clip_bottom = 46.4;

// Length of the top of the trapezoid part of the clip
clip_top = 24;

// Height of the trapezoid part of the clip
clip_height = 50.4;

// Width of the base part
base_width = 21.5;

// Depth of the base
base_depth = 4.2;

// Add extra depth to make the clip stand out
extra_depth = 11;

// Space at top where base doesn't extend
top_gap = 3.75;

// How far down fom the top of the clip is the latch indent
latch_drop = 8.25;

// How wide is the latch indent
latch_width = 12.5;

// How deep into the clip does the latch indent extend
latch_depth = 2.5;

/*[Hidden]*/
// Height of the trapezoid on the chamfered fce
scale_height = 42;

// How thick is the straight part of the trapezoid
clip_thickness = 2.5;

// How thick is the chamfered part of the trapezoid
chamfered_thickness = 1.75;

// Calculate the extrusion sale factor to make the chamfered section
scale_fac = scale_height/clip_height;

// Calculate the height of the base
base_height = clip_height - top_gap;

/*[Hidden]*/
// distance to jump over from axis to start top 
top_start = (clip_bottom-clip_top)/2;

//define the points for trapezoid part of the clip
CLIP = [[0,0],[clip_bottom,0],[top_start+clip_top,clip_height],[top_start, clip_height]];

// matrix to move the clipCLIP to the center;
CENTER =   [[-clip_bottom/2,-clip_height/2],
            [-clip_bottom/2,-clip_height/2],
            [-clip_bottom/2,-clip_height/2],
            [-clip_bottom/2,-clip_height/2]];

// center the CLIP
CENTERED_CLIP = CLIP+CENTER;

/*[Rail Parameters]*/
/*******************************************************************************
*             short
*         |-- _width--|
*         |           |    
*          -----------    ---
*    --- /             \   |
*short_ |               |  rail_depth 
* depth |               |  |
*    --- ---------------  ---
*       |    rail_      |
*       |--- width ---- |
*
*******************************************************************************/
// Width of Packout rail
rail_width = 29.5;

// Depth of Packout rail
rail_depth = 13.5;

// Width of inside part of rail
short_width = 16;

// Depth of packout rail before angle cut
short_depth = 7;

// Thickness of clamp walls
wall_thickness = 11;

// how much to round off the corners of the rail clamp
rail_chamfer = 4;

// Height of clamp
rail_clamp_height = 25;

// What is the wrench size of the hex nut to be pressed
hex_nut_size = 8.62;

// Adjust the pressed hex nut fit
hex_nut_fit = 1.01;

// How deep should the hex nut sit in the strut nut?
hex_nut_depth = 4;

gap = (rail_width-short_width)/2;

RAIL_EXTRUSION= [[0,0],
                [rail_width,0],
                [rail_width,short_depth],
                [short_width+gap,rail_depth],
                [gap,rail_depth],
                [0,short_depth]];

RAIL_CENTER=[[-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2],
            [-rail_width/2,-rail_depth/2]];

RAIL_PROFILE = RAIL_EXTRUSION+RAIL_CENTER;

/*[Screw Parameters]*/
/*******************************************************************************
*
*      |----------|<-- counter_diameter
* ----  ---------- ---
*   |   \        /  |
*   |    \      /   | <--counter_depth
* total_  |    |   ---       
* depth   |    |
*   |     |    |
*   |     |    |
* -----    ----
*         |----|<-- hole_dia
*
*******************************************************************************/
// Do you want a counterbore or countersink?
hole_type = 0; //[0:Counterbore, 1: Countersink]

// Diameter of the holes
hole_dia = 5;

// If you are using countersinks, what angle?
countersink_angle = 82; // [60,82,90,100,110,120]

// Depth of the countersink or bore
counter_depth = 4;

// Counterbore diameter (countersink uses depth and angle)
counter_dia = 9;

/*[Connector Parameters]*/
/*******************************************************************************
* Locking Packout Foot Connector Definition
*     -------
*    (   O   )
*     -------
*   |         |
*   |---------| gear_hole_diameter 
*******************************************************************************/
// How tall is the connector on top of the shaft
connector_height = 2.1;//[1:0.1:4]

// Diameter of the gear shaft (defines size of connector)
gear_hole_dia = 22;

/*[Stamp Options]*/
/*******************************************************************************
* Stamp Options
*******************************************************************************/
// Which font do you want to use
font = "REGISTRATION PLATE UK:style=Bold";

// How big you want the text
font_size = 8;//[0:0.5:8]

// How thick the font is (back to front)
font_thickness = 0.6;

/*[Hidden]*/
/*******************************************************************************
* Calculations
*******************************************************************************/
// enum type 
UNIVERSAL_CLIP=0; RAIL_CLAMP=1;

// Change sign of the X part of the vector
NEG_X =[[-1,0,0],[0,1,0],[0,0,1]];

// Calculate the point to point diameter of the pressed nut
pt_to_pt_hex_nut_dia = 2*(hex_nut_size/2)/cos(30)* hex_nut_fit;

// Calculate the diameter of the countersink hole
bottom_thickness = 3;
countersink_dia = 2*tan(countersink_angle/2)*bottom_thickness;

// add a tiny amount to cuts to make them render properly
$dl=0.01;

// calculate the position of the face 
face_pos = clip_thickness+chamfered_thickness+base_depth/2;

// calculate the extrusion scale factor to get the chamfered clamp sections 
clamp_scale_x = (rail_width+2*wall_thickness-2*rail_chamfer)/(rail_width+2*wall_thickness);
clamp_scale_y = (rail_depth+2*wall_thickness-2*rail_chamfer)/(rail_depth+2*wall_thickness);
CLAMP_SCALE = [clamp_scale_x,clamp_scale_y];

/*******************************************************************************
* Make it
*******************************************************************************/

if(type == UNIVERSAL_CLIP)
    translate([0,0,clip_height/2])
        make_universal_clip();
else if(type == RAIL_CLAMP)  
        make_rail_clamp();

/*******************************************************************************
* Make the rail clamp
*******************************************************************************/
module make_rail_clamp(){
    difference(){
        union(){
            // extrude rail clamp
            translate([0,0,rail_chamfer])    
            rotate([0,180,0])
            linear_extrude(height = rail_chamfer,$fn=100,scale =CLAMP_SCALE,slices=100,twist=0) 
                    offset(delta = wall_thickness) 
                        polygon(RAIL_PROFILE);    
                
            // extrude rail clamp
            translate([0,0,rail_chamfer])    
            linear_extrude(height = rail_clamp_height-2*rail_chamfer,$fn=100) 
                    offset(delta = wall_thickness) 
                        polygon(RAIL_PROFILE);
                
            // extrude rail clamp top
            translate([0,0,rail_clamp_height-rail_chamfer])    
            linear_extrude(height = rail_chamfer,$fn=100,scale = CLAMP_SCALE,slices=100,twist=0) 
                    offset(delta = wall_thickness) 
                        polygon(RAIL_PROFILE);
        }// end union
        translate([0,0,-$dl])
        linear_extrude(height = rail_clamp_height+2*$dl,$fn=100) 
            polygon(RAIL_PROFILE);
        
        // Cut screw holes
        translate([-(rail_width+wall_thickness)/2,0,rail_clamp_height/2])
        rotate([90,0,0])
        cylinder(d=hole_dia,h=rail_depth+2*wall_thickness+$dl,center = true,$fn = 50);
            
        translate([(rail_width+wall_thickness)/2,0,rail_clamp_height/2])
        rotate([90,0,0])
        cylinder(d=hole_dia,h=rail_depth+2*wall_thickness+$dl,center = true,$fn = 50);
          
        // cut nut holes
        translate([-(rail_width+wall_thickness)/2,rail_depth/2+2,rail_clamp_height/2])
        rotate([90,30,0])
        cylinder(d=pt_to_pt_hex_nut_dia,h=rail_depth,center = true,$fn = 6);
            
        translate([(rail_width+wall_thickness)/2,rail_depth/2+2,rail_clamp_height/2])
        rotate([90,30,0])
        cylinder(d=pt_to_pt_hex_nut_dia,h=rail_depth,center = true,$fn = 6);
    
    // cut into two pieces
    translate([0,-rail_depth/2,rail_clamp_height/2])
    cube([rail_width+2*wall_thickness+$dl,1,rail_clamp_height+$dl],center = true);
    
    // hide flat peice
    translate([0,-rail_depth/2-6,rail_clamp_height/2])
    cube([rail_width+2*wall_thickness+$dl,12,rail_clamp_height+$dl],center = true);

    // stamp version
    translate([0,rail_depth/2+wall_thickness-font_thickness,rail_clamp_height/2])
    rotate([90,0,180])
    stamp_version(version, font_size, font_thickness);

    } // end difference
} //end module
/*******************************************************************************
* Module for making the locking foot version of the clip
*******************************************************************************/
module make_universal_clip(){
    union(){  
        difference(){
            make_clip();
            
            // Stamp version
            translate([0,face_pos-font_thickness,-3])
                rotate([90,0,180])
                    stamp_version(version, font_size, font_thickness);
 
            // make screw holes (countersink to 9mm)
            if(rail_mount_holes){
                screw_offset = (rail_width + wall_thickness)/2;
                translate([-screw_offset,-extra_depth/2-base_depth/2+6,(base_height-rail_clamp_height-top_gap)/2])
                rotate([-90,0,0])
                screw_hole();
                
                translate([screw_offset,-extra_depth/2-base_depth/2+6,(base_height-rail_clamp_height-top_gap)/2])
                rotate([-90,0,0])
                screw_hole();
            } //end if
            
            // drill wall mount holes
            if(wall_mount_holes){
                translate([0,face_pos-latch_depth,-14.4])
                    rotate([-90,0,0])
                        screw_hole();

                translate([0,face_pos-latch_depth,11])
                    rotate([-90,0,0])
                        screw_hole();  
                    
                // make the connector
                if (foot_connector)
                    translate([0,-$dl-extra_depth,-14.4])
                        rotate([90,0,0])
                            connector();
                
                if(set_screw_nuts){
                    // cut nut holes
                    back_pos = -$dl-extra_depth-base_depth/2;
                    translate([0,back_pos+2,-14.4])
                    rotate([90,0,0])
                    cylinder(d=pt_to_pt_hex_nut_dia,h=4,center = true,$fn = 6);
            
                    translate([0,back_pos+2,11])
                    rotate([90,0,0])
                    cylinder(d=pt_to_pt_hex_nut_dia,h=4,center = true,$fn = 6);
                }//end if

            }//end if         
        } //end difference
    }//end union
} // end module

/*******************************************************************************
* Module for making the clip
*******************************************************************************/
module make_clip(){
    
cut_width = 20;
slope = 10;
SLOPE_CUT = [cut_width,slope,slope];
SLOPE_POS = [(base_width+cut_width)/2,-base_depth/2,(base_height)/2-top_gap/2];

ZERO_LATCH_POS = [0,face_pos-latch_depth/2+$dl,(clip_height-latch_width)/2];
LATCH_HOLE_DEF = [latch_width,latch_depth,latch_width];

    difference(){
        union(){
             //make chamfered clip top
            translate([0,base_depth/2+clip_thickness,0])
            rotate([90,0,180])
            linear_extrude(height = chamfered_thickness,scale = scale_fac) 
                polygon(CENTERED_CLIP);
            
            //make clip body
            translate([0,base_depth/2,0])
            rotate([90,0,180])
            linear_extrude(height = clip_thickness) 
                polygon(CENTERED_CLIP);
            
            //make base
            translate([0,0,-top_gap/2])
            cube([base_width,base_depth,base_height],center=true);
            difference(){
                // make pentagon shape base
                h=clip_height-top_gap;
                stub_height = h-rail_clamp_height;
                translate([0,-extra_depth/2-base_depth/2,(base_height-rail_clamp_height-top_gap)/2])
                hull(){
                    cube([rail_width+2*wall_thickness,extra_depth,rail_clamp_height],center = true);
                    
                    translate([0,0,-rail_clamp_height/2-stub_height/2])
                    cube([base_width,extra_depth,stub_height],center = true);
                } //end hull
                        
                // slope top               
                translate(SLOPE_POS)
                rotate([45,0,0])
                cube(SLOPE_CUT,center = true);
                
                translate(SLOPE_POS*NEG_X)
                rotate([45,0,0])
                cube(SLOPE_CUT,center = true);
            }//end difference       
        
        } //end union
               
        // make square latch hole
        translate(ZERO_LATCH_POS - [0,0,latch_drop])
        cube(LATCH_HOLE_DEF,center=true);
        
    } //end difference
} //end module

/*******************************************************************************
* Module for drilling counterbored or countersuck screw holes
*******************************************************************************/
module screw_hole(){
// Stick above the plane to get a good cut in uneven surfaces    
extra_height = 3;
    
// Calculate the diameter of the countersink hole
countersink_dia = 2*tan(countersink_angle/2)*(counter_depth+extra_height);

    union(){
        translate([0,0,-10])cylinder(d=hole_dia,h=20,center=true,$fn=25);
        if(hole_type == 0)
            translate([0,0,-counter_depth/2+extra_height/2])
            cylinder(d=counter_dia,h=counter_depth+extra_height,center=true,$fn=25);
        else
            translate([0,0,-counter_depth/2+extra_height/2])
            cylinder(d2=countersink_dia+hole_dia,d1=hole_dia,h=counter_depth+extra_height,center=true,$fn=25);
    }// end union
} // end module

/*******************************************************************************
* Module to create the connector hole for the locking packout foot
*******************************************************************************/
module connector(){

CUTOFF_DEF = [20,10,connector_height+$dl];
cutoff_pos = 10.1;
    
    translate([0,0,connector_height/2])
    difference(){
        cylinder(d=gear_hole_dia,h=connector_height,center=true,$fn=100);
        translate([0,cutoff_pos,0])
            cube(CUTOFF_DEF,center=true);
        translate([0,-cutoff_pos,0])
            cube(CUTOFF_DEF,center=true);
    } // end difference
} //end module

/*******************************************************************************
* Module for putting the version number on the print
*******************************************************************************/
module stamp_version(version_text,size,thickness){
    
    color("white")
    linear_extrude(height = thickness)
        text(text = version_text, font = font, size = size, valign="center",halign = "center");
} // end module
